; Movement routines
; by David Phillips <david@acz.org>
; Assembly Coder's Zenith -- http://www.acz.org/

; Directions:  0 = up, 1 = down, 2 = left, 3 = right
; Character: Y (0-127), X (0-63), Direction (0-3), Animation Counter (0-ANIM)

;============================================================================

; hl = address of x or y coord
; bc = coords loaded (x, y)
; returns: carry set = collision
TryMoveUp:
 dec c                  ; check tile above
 call CheckSpriteVert   ; check for collision
 ret c                  ; if we collided, return
 dec (hl)               ; move up
 ret                    ; return
TryMoveDown:
 ld a,8                 ; check tile below
 add a,c                ; sprites are 8 pixels
 ld c,a                 ; save coord
 call CheckSpriteVert   ; check for collision
 ret c                  ; if we collided, return
 inc (hl)               ; move down
 ret                    ; return
TryMoveLeft:
 dec b                  ; check tile to the left
 call CheckSpriteHorz   ; check for collision
 ret c                  ; if we collided, return
 dec (hl)               ; move left
 ret                    ; return
TryMoveRight:
 ld a,8                 ; check tile to the right
 add a,b                ; sprites are 8 pixels
 ld b,a                 ; save coord
 call CheckSpriteHorz   ; check for collision
 ret c                  ; if we collided, return
 inc (hl)               ; move right
 ret                    ; return

; check sprite and sprite to the right
; bc = sprite location to check (x, y)
; returns: carry = collision
CheckSpriteVert:
 call GetTile           ; calc address of tile on map
 call CheckSolid        ; is it solid?
 jr c,CheckSpriteVertP  ; setup other pointer
 ld a,b                 ; get x coord
 and %00000111          ; is it alligned?
 ret z                  ; then we're done
 inc de                 ; check sprite to the right
 call CheckSolid        ; is it solid?
 ret                    ; we're done
CheckSpriteVertP:
 inc de
 ret

; check sprite and sprite to the bottom 
; bc = sprite location to check (x, y)
; returns: carry = collision
CheckSpriteHorz:
 call GetTile           ; calc address of tile on map
 call CheckSolid        ; is it solid?
 ret c                  ; then we're done
 jr c,CheckSpriteHorzP  ; setup other pointer
 ld a,c                 ; get y coord
 and %00000111          ; is it alligned?
 ret z                  ; then we're done
 ex de,hl               ; do calculations in hl
 ld bc,16               ; get tile to the bottom
 add hl,bc              ; calc offset
 ex de,hl               ; swap back
 call CheckSolid        ; is it solid?
 ret                    ; we're done
CheckSpriteHorzP:
 ex de,hl
 ld bc,16
 add hl,bc
 ex de,hl
 ret

; de = address of tile to check
; returns: carry = tile is solid
CheckSolid:
 ld a,(de)              ; load the tile 
 cp SOLID               ; is it clear?
 ccf                    ; reverse check
 ret                    ; return

; hl = coords in (y, x) format
GetTileHL:
 ld c,(hl)              ; get y coord
 inc hl                 ; next byte
 ld b,(hl)              ; get x coord
 dec hl                 ; back to last byte 

; bc = coords to get (x, y)
; returns: de = pointer to map tile
GetTile:
 ex de,hl               ; hl will be saved in de
 push bc                ; save bc
 srl b                  ; divide x by 2
 srl b                  ; / 4
 srl b                  ; / 8
 ld a,c                 ; get y coord
 and %11111000          ; clear lower 3 bits
 add a,a                ; * 2
 add a,b                ; calc offset
 ld l,a                 ; load into low byte
 ld h,0                 ; clear high byte
 ld bc,Map              ; get tile from the current map
 add hl,bc              ; compute offset in map
 ex de,hl               ; need to return in de
 pop bc                 ; restore bc
 ret                    ; return

; hl = address of coords (x, y)
; returns: a = number of tile
;          bc = coords loaded
GetTileNum:
 ld a,(hl)
 ld c,a
 and %11111000
 add a,a
 ld d,a
 inc hl
 ld a,(hl)
 ld b,a
 rra
 rra
 rra
 and %00001111
 add a,d
 ret

; hl = address of coords (x, y)
; returns: a = number of tile
;          bc = coords loaded and centered
GetTileNumCenter:
 ld a,(hl)
 add a,3
 ld c,a
 and %11111000
 add a,a
 ld d,a
 inc hl
 ld a,(hl)
 add a,3
 ld b,a
 rra
 rra
 rra
 and %00001111
 add a,d
 ret

; bc = coords to get (x, y)
; returns: de = pointer to map tile
GetTileCent:
 call CenterCoords
 jr GetTile 

; bc = coords to center (x, y)
; returns: bc = centered coords (x+3, y+3)
CenterCoords:
 inc b
 inc b
 inc b
 inc c
 inc c
 inc c
 ret

; hl = pointer to map tile
; returns: bc = coords of map tile
GetCoords:
 push hl
 ld bc,($ffff-Map)+1
 add hl,bc
 ld a,%11110000
 and l
 rra
 ld c,a
 ld a,%00001111
 and l
 rla
 rla
 rla
 ld b,a
 pop hl
 ret

; hl = byte to update
UpdateAnim:
 ld a,(Anim)
 or a
 ret nz
 inc (hl)
 ld a,3
 cp (hl)
 ret nz
 ld (hl),0
 ret 

; bc = (x1,y1), de = (x2,y2)
; returns: no carry = collision
CheckCollision:
 ld a,d
 add a,7
 cp b
 ret c
 ld a,b
 add a,7
 cp d
 ret c
 ld a,e
 add a,7
 cp c
 ret c
 ld a,c
 add a,7
 cp e
 ret 

; in: bc = point, de = coords
; returns: no carry = collision
CheckPoint:
 ld a,d
 add a,7
 cp b
 ret c
 ld a,e
 add a,7
 cp c
 ret c
 ld a,b
 cp d
 ret c
 ld a,c
 cp e
 ret

; hl = coords (then dir, anim)
; de = start of sprites
GrayLoadDrawSprite:
 ld c,(hl)
 inc hl
 ld b,(hl)
 inc hl
; hl = dir byte (followed by anim byte)
; de = start of sprites
; bc = sprite draw location (x, y)
GrayDrawSprite:
 push bc                ; save draw location
 ld a,(hl)              ; load dir byte
 rlca                   ; each set is 72 bytes (* 2)
 rla                    ; * 4
 rla                    ; * 8
 ld c,a                 ; save (* 8)
 rla                    ; * 16
 rla                    ; * 32
 rla                    ; * 64
 add a,c                ; (* 64) + (* 8) = * 72
 ld c,a                 ; save (* 72)
 inc hl                 ; move to anim byte
 ld a,(hl)              ; get anim byte
 ld h,0                 ; clear high byte
 rla                    ; sprites are 24 bytes (* 2)
 rla                    ; * 4
 rla                    ; * 8
 ld b,a                 ; save (* 8)
 rla                    ; * 16
 add a,b                ; (* 16) + (* 8) = * 24
 add a,c                ; calc total offset
 jr nc,GrayDrawSpriteNC ; skip carry
 inc h                  ; increase high byte
GrayDrawSpriteNC:
 ld l,a                 ; load into low byte
 add hl,de              ; calculate correct sprite
 pop bc                 ; restore draw location
 jp GraySpriteClip      ; draw sprite and return

.end
