; Zelda 86 by the Assembly Coder's Zenith
; The Eye Of Ganon
; http://www.acz.org/
;
; Programmer: David Phillips <david@acz.org>
; Graphics:   Brett Barwick
;             Dave Sheltema
; Maps:       Jeremy Goetsch <jeremy@acz.org>
;             Matt Johnson   <matt2000@dogtech.com>
;
; Started:     December ??, 1998
; Last Update: November 05, 2000


#include "atom.asm"

Frame           = Vars          ; [1]       ; counter increased each frame
RedrawMap       = Vars + 1      ; [1]       ; if non-zero, redraw map at end of frame

PlayerY         = Vars + 5      ; [1]       ; player's position on map
PlayerX         = Vars + 6      ; [1]
PlayerDir       = Vars + 7      ; [1]       ; up, down, left, right
PlayerAnim      = Vars + 8      ; [1]       ; animation frame (0 - 2)

RaftDir         = Vars + 9      ; [1]       ; direction of raft (if using), -1 = not using

PlayerFlash     = Vars + 14     ; [1]       ; frames left of flashing (invincibility)

PlayerJump      = Vars + 15     ; [1]       ; height in air, bit 7 = set if going up

ButtonA         = Vars + 20     ; [1]       ; state of buttons, 0 = not pressed
ButtonB         = Vars + 21     ; [1]       ;

PlayerSwordDir  = Vars + 30     ; [1]       ; direction of sword            1 0 7
PlayerSwordF    = Vars + 31     ; [1]       ; frames left (directions)      2 * 6
                                            ;                               3 4 5
PlayerBomb      = Vars + 32     ; [1]       ; time left, 0 = inactive
PlayerBombY     = Vars + 33     ; [1]
PlayerBombX     = Vars + 34     ; [1]

PlayerBoomerT   = Vars + 35     ; [1]       ; time left, 0 = inactive, bit 7 = returning
PlayerBoomerDir = Vars + 36     ; [1]       ; direction of boomerang (%xxxxRLDU)
PlayerBoomerY   = Vars + 37     ; [1]       ;
PlayerBoomerX   = Vars + 38     ; [1]       ;

PlayerArrowDir  = Vars + 39     ; [1]       ; direction of arrow, -1 = inactive
PlayerArrowY    = Vars + 40     ; [1]       ;
PlayerArrowX    = Vars + 41     ; [1]       ;

Items           = Vars + 42     ; [30]      ; 10 x (item[1], coords[2]), 0 = not active

Enemies         = Vars + 100    ; [80]      ; 10 x (number[1], coords[2], dir[1],
                                            ;       anim[1], life[1], special[2])

People          = Vars + 180    ; [8]       ; 4 x (sprite number, text string number)

EnemyProj       = Vars + 188    ; [30]      ; 10 x (dir[1], coords[2]), -1 = inactive

_@@VARS_END     = Vars + 218

ShellTitle:
 .db "Zelda 86 by the ACZ",0

ProgStart:

LoadNumMaps:
 ld hl,MapVar-1
 rst 20h
 rst 10h
 ret c
 call _ex_ahl_bde
 call _inc_ptr_ahl
 call _get_word_ahl
 push de
 call _load_ram_ahl
 pop de
 ld a,d
 call _ram_page_1
 ld (NumMaps),a

LoadData:
 ld hl,ZeldaDatVar-1
 rst 20h
 rst 10h
 ret c
 call _ex_ahl_bde
 call _get_word_ahl
 call _set_abs_src_addr
 ld a,1
 ld hl,ZeldaDat-$8000
 call _set_abs_dest_addr
 sub a
 ex de,hl
 call _set_mm_num_bytes
 call _mm_ldir
 call _ram_page_1

 call _flushallmenus
 res indicRun,(iy+indicflags)

 call OpenGray

ShowTitle:
 call DisplayTitle
 call ShowGray
 call Pause

DoMenu:
 call DisplayTitle
 ld hl,GrayMem1+(16*35)
 ld de,GrayMem1+(16*35)+1
 ld bc,16*29
 ld (hl),0
 ldir
 ld hl,GrayMem2+(16*35)
 ld de,GrayMem2+(16*35)+1
 ld bc,16*29
 ld (hl),0
 ldir
 ld bc,(16*17)+5
 ld hl,MenuTextNew
 call PrintTextString
 ld bc,(16*26)+5
 ld hl,MenuTextLoad
 call PrintTextString

DoMenuLoop:
 ld de,SprPlayer
 ld hl,MenuSpriteInfo
 call GrayLoadDrawSprite
 call ShowGray
 ld hl,MenuSpriteInfo+3
 ld a,(hl)
 cp 2
 jr nz,DoMenuSkipAnim
 ld (hl),-1 
DoMenuSkipAnim:
 inc (hl) 
 ld b,30
DoMenuLoopDelay:
 halt
 call _get_key
 or a
 jr nz,DoMenuLoopDelayDone
 djnz DoMenuLoopDelay
DoMenuLoopDelayDone:
 cp K_EXIT
 jp z,Exit
 cp K_SECOND
 jr z,DoMenuSelect
 cp K_UP
 jr z,DoMenuToggle
 cp K_DOWN
 jr z,DoMenuToggle 
 cp K_ENTER
 jr z,DoMenuSelect
 jr DoMenuLoop
DoMenuToggle:
 ld hl,MenuSpriteInfo
 ld a,(hl)
 cp 41
 jr nz,DoMenuToggleNew
 ld (hl),50
 ld a,' '
 ld bc,(16*17)+3
 call PrintChar 
 jr DoMenuLoop
DoMenuToggleNew:
 ld (hl),41
 ld a,' '
 ld bc,(16*26)+3
 call PrintChar 
 jr DoMenuLoop
DoMenuSelect:
 ld hl,MenuSpriteInfo
 ld a,(hl)
 cp 41
 jr z,StartNewGame
 jr LoadGame

DisplayTitle:
 ld hl,TitlePicLightLayer
 ld de,GrayMem2
 call DispRLE
 ld hl,TitlePicDarkLayer
 ld de,GrayMem1
 call DispRLE
 ret

; y = 41 or 50
MenuSpriteInfo:
 .db 50, 24, 1, 0

MenuTextNew:
 .db "New Game",0

MenuTextLoad:
 .db "Load Game",0

LoadGame:
 ld a,(PlayerMaxLife)
 or a
 jr z,StartNewGame
 
 ld hl,(CurrMapEnterCoords)
 ld (PlayerY),hl
 ld hl,PlayerY
 call GetTileNumCenter
 ld (CurrTile),a
 ld a,(CurrMap)
 call LoadMapNum
 call GrayDrawMap

 jr PlayGame

StartNewGame:
 ld hl,MapVar-1
 rst 20h
 rst 10h
 call _ex_ahl_bde
 call _inc_ptr_ahl
 call _get_word_ahl
 call _get_word_ahl
 add hl,de
 adc a,0
 call _set_abs_src_addr
 ld hl,Chests
 sub a
 call _set_abs_dest_addr
 ld hl,CHESTS*3
 call _set_mm_num_bytes
 call _mm_ldir 
 call _ram_page_1

 call ClearChangeList

 ld hl,(96<<8)+24
 ld (PlayerY),hl

 ld hl,0
 ld (PlayerRupees),hl
 ld a,6
 ld (PlayerLife),a
 ld (PlayerMaxLife),a
 sub a
 ld (PlayerItems),a
 ld (PlayerSpecial),a
 ld (ButtonBEquipped),a 
 ld (PlayerKeys),a

 ld a,2
 call LoadMapNum
 call GrayDrawMap

PlayGame:
 ld a,1
 ld (PlayerAnim),a

 sub a
 ld (Anim),a
 ld (PlayerDir),a
 ld (Frame),a
 ld (RedrawMap),a
 ld (ButtonA),a
 ld (ButtonB),a
 ld (PlayerSwordF),a 
 ld (PlayerBomb),a
 ld (PlayerFlash),a
 ld (PlayerJump),a
 dec a
 ld (RaftDir),a

MainLoop:
 ld hl,Anim
 ld a,(hl)
 inc a
 cp 3
 jr nz,SkipResetAnimCounter
 sub a
SkipResetAnimCounter:
 ld (hl),a

AnimateMap:
 ld a,(Frame)
 and %00000111
 jr nz,AnimateMapSkip
 ld de,Map
 ld c,0
AnimateMapLoop:
 ld a,(de)
 ld b,AnimateMapListSize
 ld hl,AnimateMapList
AnimateMapSearch:
 cp (hl)
 inc hl
 jr nz,AnimateMapSearchNext
 ld a,(hl)
 ld (de),a
 call AnimateMapSprite
 jr AnimateMapNext
AnimateMapSearchNext:
 inc hl
 djnz AnimateMapSearch
AnimateMapNext:
 inc de
 inc c
 bit 7,c
 jr z,AnimateMapLoop
 jr AnimateMapDone
AnimateMapSprite:
 push hl
 push de
 push bc
 ld h,0
 ld b,h
 ld l,a
 add hl,hl
 add hl,hl
 add hl,hl
 add hl,hl
 ld ix,MapSprites
 ex de,hl
 add ix,de
 ld a,c
 and %11110000
 add a,a
 ld h,b
 ld l,a
 add hl,hl
 add hl,hl
 ld a,c
 and %00001111
 ld c,a
 add hl,bc
 ex de,hl
 ld hl,Buffer2
 add hl,de
 ex de,hl
 ld bc,Buffer1
 add hl,bc
 ld bc,16
 ld a,8
AnimateMapSpriteL:
 ld (_@AnimateMapSpriteC),a
 ld a,(ix+0)
 ld (hl),a
 add hl,bc
 ex de,hl
 ld a,(ix+8)
 ld (hl),a
 add hl,bc
 ex de,hl
 inc ix
_@AnimateMapSpriteC =$+1
 ld a,0
 dec a
 jr nz,AnimateMapSpriteL
 pop bc
 pop de
 pop hl
 ret 
AnimateMapSkip:
 ld b,5
Delay:
 halt
 djnz Delay
AnimateMapDone:
 call CopyBuffers

MovePlayer:
 ld hl,RaftDir
 ld a,(hl)
 cp $ff
 jp nz,DoRaft
MovePlayerMove:
 ld a,%01111110
 out (1),a
 nop \ nop
 in a,(1)
MovePlayerDown:
 rra
 jp nc,MoveDown
MovePlayerLeft:
 rra
 jp nc,MoveLeft
MovePlayerRight:
 rra
 jp nc,MoveRight
MovePlayerUp:
 rra
 jp nc,MoveUp
OtherKeys:
 ld a,%00111111
 out (1),a
 nop \ nop
 in a,(1)
 bit 5,a
 jp z,HitButtonA
 jp NoHitButtonA
OtherKeys2:
 ld a,%01011111
 out (1),a
 nop \ nop
 in a,(1)
 rla
 jp nc,HitButtonB
 jp NoHitButtonB
OtherKeys3:
 call _get_key
 cp K_EXIT
 jp z,Exit
 cp K_MORE
 jp z,Options
 cp K_PLUS
 jp z,IncContrast
 cp K_MINUS
 jp z,DecContrast
; cp K_SQUARE
; jr z,ItemCheat
 jp EndInput

#ifdef 1
ItemCheat:
 ld a,6
 call SetPlayerItems
 ld hl,PlayerSpecial
 set 0,(hl)
 ld a,24
 ld (PlayerMaxLife),a
 ld (PlayerLife),a
 inc a
 ld (PlayerArrows),a
 ld (PlayerBombs),a
 ld (PlayerKeys),a
 ld hl,500
 ld (PlayerRupees),hl 
 ld hl,CheaterText
 call PrintText
 jp EndInput
CheaterText:
 .db "Cheater!!!",0
#endif

MoveUp:
 call MovePlayerSetup
 ld (hl),0              ; set dir to up
 inc hl
 call UpdateAnim
 ex de,hl
 call CheckWarpUp
 jr c,MoveUpDone
 call TryMoveUp
 call c,PlayerSolid
 call CheckTile
MoveUpDone:
 pop af
 jp OtherKeys

MoveDown:
 call MovePlayerSetup
 ld (hl),1              ; set dir to down
 inc hl
 call UpdateAnim
 ex de,hl
 call CheckWarpDown
 jr c,MoveDownDone
 call TryMoveDown
 call c,PlayerSolid
 call CheckTile
MoveDownDone:
 pop af
 jp MovePlayerLeft

MoveLeft:
 call MovePlayerSetup
 ld (hl),2              ; set dir to left
 inc hl
 call UpdateAnim
 ex de,hl
 inc hl
 call CheckWarpLeft
 jr c,MoveLeftDone
 call TryMoveLeft
 dec hl
 call c,PlayerSolid
 call CheckTile
MoveLeftDone:
 pop af
 jp MovePlayerRight

MoveRight:
 call MovePlayerSetup
 ld (hl),3              ; set dir to right
 inc hl
 call UpdateAnim
 ex de,hl
 inc hl
 call CheckWarpRight
 jr c,MoveRightDone
 call TryMoveRight
 dec hl
 call c,PlayerSolid
 call CheckTile
MoveRightDone:
 pop af
 jp MovePlayerUp

HitButtonA:
 ld hl,ButtonA
 sub a
 or (hl)
 jr z,JustHitButtonA
HoldButtonA:            ; button A is held
 inc (hl) 
 ld hl,PlayerSwordF
 sub a
 or (hl)
 jp nz,OtherKeys2
_@HoldButtonACheck =$+1
 ld a,0
 or a
 jp nz,OtherKeys2  
 ld (hl),2
 jp OtherKeys2
JustHitButtonA:         ; button A just hit
 ld a,(PlayerItems)
 cp 2
 jp c,OtherKeys2
 sub a
 ld (_@DrawPlayerShieldTest),a
 inc (hl)
 call SwingSword
 jp OtherKeys2
NoHitButtonA:
 ld hl,ButtonA
 sub a
 or (hl)
 jp z,OtherKeys2
ReleaseButtonA:         ; button A released
 ld a,(_@HoldButtonACheck)
 or a
 ld a,0
 ld (_@HoldButtonACheck),a
 ld a,(hl)
 ld (hl),0
 jp nz,OtherKeys2
 cp SWORDCHARGE
 call nc,ChargeSword
 jp OtherKeys2

HitButtonB:
 ld hl,ButtonB
 sub a
 or (hl)
 jr z,JustHitButtonB
HoldButtonB:            ; button B is held
 inc (hl) 
 jp OtherKeys3
JustHitButtonB:         ; button B just hit
 inc (hl)
 ld hl,OtherKeys3
 push hl
 ld a,(ButtonBEquipped)
 or a
 ret z
 dec a
 jp z,DropBomb
 dec a 
 jp z,ThrowBoomerang
 dec a
 jp z,MakeLinkJump
 dec a
 jp z,ShootArrow
 ret
NoHitButtonB:
 ld hl,ButtonB
 sub a
 or (hl)
 jp z,OtherKeys3
ReleaseButtonB:         ; button B released
 ld (hl),0
 jp OtherKeys3

IncContrast:
 call ContrastUp
 jp EndInput

DecContrast:
 call ContrastDown
 jp EndInput

MovePlayerSetup:
 ld h,a
 ex (sp),hl
 push hl
 ld hl,PlayerX
 ld b,(hl)
 dec hl
 ld c,(hl)
 ex de,hl
 ld hl,PlayerDir
 ret

DoRaft:
 ld hl,PlayerY
 or a
 jr z,DoRaftUp
 dec a
 jr z,DoRaftDown
 dec a
 jr z,DoRaftLeft
DoRaftRight:
 inc hl
 inc (hl)
 call CheckWarpRight 
 ld hl,RaftRight
 jr DrawRaft
DoRaftUp:
 dec (hl)
 call CheckWarpUp
 ld hl,RaftUp
 jr DrawRaft
DoRaftDown:
 inc (hl)
 call CheckWarpDown
 ld hl,RaftDown
 jr DrawRaft
DoRaftLeft:
 inc hl
 dec (hl)
 call CheckWarpLeft
 ld hl,RaftLeft
 jr DrawRaft
 
DrawRaft:
 ld bc,(PlayerY)
 call GraySpriteClip
 ld hl,PlayerY
 call GetTileNum
 ld d,a
 ld a,(CurrTile)
 cp d
 jr z,DrawRaftDone
 ld a,d
 ld (CurrTile),a
 call GetTile
 ld a,(de)
 cp DOCK
 jr nz,DrawRaftDone
 ld a,(RaftDir)
 or a
 jr z,FixRaftUp
 cp 2
 jr z,FixRaftLeft
DrawRaftFixed:
 ld a,-1
 ld (RaftDir),a
 ld hl,(PlayerY)
 ld (CurrMapEnterCoords),hl
DrawRaftDone: 
 jp DrawPlayerDone

FixRaftUp:
 ld hl,PlayerY
 ld a,(hl)
 sub 8
 ld (hl),a
 jr DrawRaftFixed
 
FixRaftLeft:
 ld hl,PlayerX
 ld a,(hl)
 sub 8
 ld (hl),a
 jr DrawRaftFixed

EndInput:

DrawPlayer:
 ld bc,(PlayerY)
 ld hl,PlayerJump               ; jump code
 ld a,(hl)
 or a
 jr z,DrawPlayerDraw
 bit 7,a
 jr z,DrawPlayerFall
 inc a
 bit 3,a
 jr z,DrawPlayerJumpOk
 and %01111111 
DrawPlayerJumpOk:
 ld (hl),a
 and %01111111
 ld (_@DrawPlayerShadowAdjust),a 
 jr DrawPlayerShadow
DrawPlayerFall:
 ld (_@DrawPlayerShadowAdjust),a
 dec a
 ld (hl),a
 jr nz,DrawPlayerShadow
 push bc
 call CenterCoords
 call EnterNewTile
 pop bc  
DrawPlayerShadow:
 push bc
 ld hl,ShadowSpr
 call GraySpriteClip
 pop bc
 ld a,c
_@DrawPlayerShadowAdjust =$+1
 sub 0
 ld c,a                         ; end jump code
DrawPlayerDraw:
 ld hl,PlayerFlash
 ld a,(hl)
 or a
 jr z,DrawPlayerNoFlash
 dec (hl)
 and %00000001
DrawPlayerNoFlash:
 jr nz,DrawPlayerDone
 ld (_@DrawPlayerShieldCoord),bc
 ld hl,PlayerDir
 ld de,SprPlayer
 call GrayDrawSprite
_@DrawPlayerShieldTest =$+1
 ld a,1
 or a
 jr z,DrawPlayerDone
 ld a,(PlayerItems)
 or a
 jr z,DrawPlayerDone
 ld a,(PlayerDir)
 or a
 jr z,DrawPlayerDone 
_@DrawPlayerShieldCoord =$+1
 ld bc,$ffff
 dec a
 jr nz,DrawPlayerLeft
 ld hl,ShieldDownSpr
 inc c
 inc c
 jr DrawPlayerShield
DrawPlayerLeft:
 dec a
 jr nz,DrawPlayerRight
 ld hl,ShieldLeftSpr
 dec b
 jr DrawPlayerShield
DrawPlayerRight:
 ld hl,ShieldRightSpr
 inc b
DrawPlayerShield:
 call GraySpriteClip

DrawPlayerDone:
 call UpdateEnemies
 call UpdateItems
 call UpdateWeapons
 call UpdateEnemyProj
 call UpdateAnims

 ld hl,RedrawMap
 or (hl)
 jr z,RedrawDone
 ld (hl),0
 ld hl,Map
 call GrayDrawMap
RedrawDone:
 call ShowGray

EndMain:
 ld hl,Frame
 inc (hl)
 ld a,(PlayerLife)
 dec a
 cp 200
 jp c,MainLoop

PlayerDead:
 call ClearBuffers
 call CopyBuffers
 ld bc,4
 ld hl,GameOverText
 call PrintTextString
 call ShowGray
 call Pause
 ld hl,PlayerLife
 ld (hl),1

Exit:
 call CloseGray
 call SaveGame
 sub a
 ld (_asapvar+1),a
 jp _clrWindow

SaveGame:
 ld hl,ZeldaPrgmVar-1
 rst 20h
 rst 10h
 call _ex_ahl_bde
 ld de,GameSaveData-_asm_exec_ram+4
 add hl,de
 adc a,0
 call _set_abs_dest_addr
 sub a
 ld hl,GameSaveData
 call _set_abs_src_addr
 ld hl,GameSaveDataSize
 call _set_mm_num_bytes
 call _mm_ldir
 ret

GameOverText:
 .db "Game Over",0

;============================================================================
MakeLinkJump:
 ld hl,PlayerJump
 ld a,(hl)
 or a
 ret nz
 ld (hl),%10000000
 ret

; a = item level
SetPlayerItems:
 ld hl,PlayerItems
 cp (hl)
 ret c
 ld (hl),a
 ret

; bc = coords of destination tile
PlayerSolid:
 push hl
 push de
 call GetTile
 pop bc
 call TryRaft
 call TalkToPerson
 call OpenDoor
 call OpenChest
 pop hl
 ret

; de = first byte, bc = second byte
TryRaft:
 ld a,(de)
 cp WATER
 ret nz
 push bc
 push de
 ld bc,(PlayerY)
 call GetTile
 ld a,(de)
 pop de
 pop bc
 cp DOCK
 ret nz
 ld a,(PlayerSpecial)
 bit 0,a
 ret z
 ld a,(PlayerDir)
 ld (RaftDir),a
 ret 

; de = first byte, bc = second byte
TalkToPerson:
 push de
 push bc
 ld hl,People
 ld a,4
TalkToPersonLoop:
 ld (_@TalkToPersonLoopC),a
 ld a,(de)
 cp (hl)
 jr z,TalkToPersonFound
 ld a,(bc)
 cp (hl)
 jr z,TalkToPersonFound
 inc hl
 inc hl
_@TalkToPersonLoopC =$+1
 ld a,0
 dec a
 jr nz,TalkToPersonLoop
TalkToPersonDone:
 pop bc
 pop de
 ret
TalkToPersonFound:
 inc hl
 ld a,(hl)
 call PrintString
 jr TalkToPersonDone

;============================================================================
; a fast, yet decent random number generator
; returns: a = psuedo random number
; destroyed: flags
RandomNumber:
 push bc
 ld a,r
 ld b,a
 ld a,($1234)
_@RandomNumberSeed =$-2
 ld c,a
 ld a,(bc)
 ld (_@RandomNumberSeed),a
 xor b
 sub b
 add a,c
 pop bc
 ret

Pause:
 push hl
 call _get_key
 pop hl
 cp 5           ; keys 1-4 are the arrows
 ret nc
 halt
 jr Pause

;============================================================================
#include "link.asm"

#include "enemies.asm"

#include "text.asm"

#include "options.asm"

#include "chests.asm"

#include "doors.asm"

#include "changelist.asm"

;============================================================================
#include "zeldadat.inc"

; (old, new)
AnimateMapList:
 .db 164,165, 165,166, 166,164                  ; waterfall
 .db  81,167, 167,81                            ; torch
 .db 129,132, 132,129                           ; water
 .db 130,131, 131,130                           ; waterfall splash
 .db 104,176, 176,104                           ; person
 .db 168,169, 169,170, 170,171, 171,168         ; kid w/ bouncing ball
 .db 172,173, 173,174, 174,175, 175,172         ; teleporter
 .db 177,178, 178,179, 179,180, 180,177         ; guy w/ shovel
AnimateMapListSize = ($ - AnimateMapList) / 2

; (y, x)
ProjectileOffsetTable:
 .db 0,3, 7,3, 3,0, 3,7

; (x, y)
DirTableCoords2:
 .db 0,-2, 0,2, -2,0, 2,0

ZeldaDatVar:
 .db 8,"zeldadat"

ZeldaPrgmVar:
 .db 5,"zelda"

GameSaveData:

; 100 x (map number, offset on map, item number), 0 = open
Chests:
 .dw 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0      ; 300 bytes
 .dw 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
 .dw 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
 .dw 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
 .dw 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
 .dw 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

; 50 x (map number, offset on map, target tile), map 255 = open
ChangeList:
 .dw 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0      ; 150 bytes
 .dw 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
 .dw 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

CurrMap:                ; current map number
 .db 0                  ; [1]
CurrMapEnterCoords:     ; player's coords when map was entered
 .dw 0                  ; [2]
PlayerLife:             ; player's life (1 = half heart)
 .db 0                  ; [1]
PlayerMaxLife:          ; player's maximum life (number of half hearts)
 .db 0                  ; [1]
PlayerRupees:           ; player's money
 .dw 0                  ; [2]
PlayerKeys:             ; player's normal keys left
 .db 0                  ; [1]
PlayerBombs:            ; player's bombs
 .db 0                  ; [1]
PlayerArrows:           ; player's arrows
 .db 0                  ; [1]
PlayerItems:            ; player's items (in levels...each step includes previous)
                        ; (0 = nothing, 1 = shield, 2 = sword, 3 = bombs, 4 = boomerang, 5 = feather, 6 = bow)
 .db 0                  ; [1]
PlayerSpecial:          ; player's special items (each bit is a flag)
                        ; 0 = raft
 .db 0                  ; [1]
ButtonBEquipped:        ; current item for use (0 = none, 1 = bomb, 2 = boomerang)
 .db 0                  ; [1]

GameSaveDataSize =$-GameSaveData

_@@PRGM_END =$

.end
